// BMPLoader.cpp: Implementierung der Klasse BMPLoader
//
///////////////////////////////////////////////////////////////////////

#include"BMPLoader.h"


BMPLoader::BMPLoader()
{
	image = 0;
	imageWidth = 0;
	imageHeight = 0;
}


BMPLoader::~BMPLoader()
{
   clear();
}


bool BMPLoader::LoadBitmap(char *file)
{
	FILE *pFile = 0;
	
	// these will hold the bitmap header and file information
	BITMAPINFOHEADER bitmapInfoHeader;
	BITMAPFILEHEADER header;

   // this will be used to swap the image colors from BGR to RGB
	unsigned char textureColors = 0;

   // open the file and make sure there are no errors
	pFile = fopen(file, "rb");
	if(pFile == 0) return false;

	// read in the bitmap header info into the BITMAPFILEHEADER variable
	fread(&header, sizeof(BITMAPFILEHEADER), 1, pFile);
	
	// make sure this is a real bitmap by checking the ID
	
	if(header.bfType != BITMAP_ID)
    {
		fclose(pFile);
		return false;
	}

	// read in the second header info into the BITMAPINFOHEADER variable
	fread(&bitmapInfoHeader, sizeof(BITMAPINFOHEADER), 1, pFile);

	// save the width and height.
	imageWidth = bitmapInfoHeader.biWidth;
    imageHeight = bitmapInfoHeader.biHeight;

   
	// make sure we read a size out of it
   
	if(bitmapInfoHeader.biSizeImage == 0)
		bitmapInfoHeader.biSizeImage = bitmapInfoHeader.biWidth *
		bitmapInfoHeader.biHeight * 3;

	// place the pointer in front of where the image data starts
	fseek(pFile, header.bfOffBits, SEEK_SET);

	// dynamically create enough memory for the image
	image = new unsigned char[bitmapInfoHeader.biSizeImage];

	// error checking; make sure the memory was allocated
	if(!image)
	{
		delete[] image;
		fclose(pFile);
		return false;
	}

	// read in the image
	fread(image, 1, bitmapInfoHeader.biSizeImage, pFile);

	// bitmaps are saved in BGR format so we will make the image RGB
	for(int index = 0; index < (int)bitmapInfoHeader.biSizeImage; index+=3)
	{
		textureColors = image[index];
		image[index] = image[index + 2];
		image[index + 2] = textureColors;
	}

    // close the file and return the image
	fclose(pFile);
	return true;
}


void BMPLoader::clear()
{
   if(image)
   {
	   delete[] image;
	   image = 0;
   }
}