// Camera.cpp: Implementierung der Klasse Camera
//
///////////////////////////////////////////////////////////////////////

#include"Camera.h"


Camera::Camera()
{
   // initialize variables
   xPos = yPos = zPos = 0;
   xView = yView = zView = 0;
   xUp = yUp = zUp = 0;
   xStrafe = yStrafe = zStrafe = 0;
}


void Camera::SetCamera(float x, float y, float z, 
					   float xv, float yv, float zv,
                       float xu, float yu, float zu)
{
   // set camera values
   xPos = x;   yPos = y;   zPos = z;
   xView = xv; yView = yv; zView = zv;
   xUp = xu;   yUp = yu;   zUp = zu;
}


void Camera::UpdateCamera(float xDir, float yDir, float zDir, float dir)
{
   // move camera position and view point in the xz-plane
	xPos += xDir * dir;
   //yPos += yDir * dir;
	zPos += zDir * dir;

	xView += xDir * dir;
   //yView += yDir * dir;
	zView += zDir * dir;
}

void Camera::MoveCamera(float direction)
{
   float xLookDirection = 0, yLookDirection = 0, zLookDirection = 0;

   // calculate view vector
   xLookDirection = xView - xPos;
   yLookDirection = yView - yPos;	
   zLookDirection = zView - zPos;

   // normalize
   float dp = 1 /(float)sqrt(xLookDirection * xLookDirection + yLookDirection * yLookDirection + zLookDirection * zLookDirection);
   xLookDirection *= dp;
   yLookDirection *= dp;
   zLookDirection *= dp;

   // finally update the camera
   UpdateCamera(xLookDirection, yLookDirection, zLookDirection, direction);
}


void Camera::CalculateStrafe()
{
   float xDir = 0, yDir = 0, zDir = 0;
   float xCross = 0, yCross = 0, zCross = 0;

   // calculate view vector
   xDir = xView - xPos;
   yDir = yView - yPos;
   zDir = zView - zPos;

   // normalize
   float dp = 1 /(float)sqrt(xDir * xDir + yDir * yDir + zDir * zDir);
   xDir *= dp;
   yDir *= dp;
   zDir *= dp;

   // get the strafe direction, e.g. cross product of view vector and up direction
   xCross = (yDir * zUp) - (zDir * yUp);
   yCross = (zDir * xUp) - (xDir * zUp);
   zCross = (xDir * yUp) - (yDir * xUp);

   // finally save the result
   xStrafe = xCross;
   yStrafe = yCross;
   zStrafe = zCross;
}


void Camera::StrafeCamera(float direction)
{
   // calculate the strafe
   CalculateStrafe();
   // finally update the camera
   UpdateCamera(xStrafe, yStrafe, zStrafe, direction);
}


void Camera::RotateCamera(float AngleDir, float xSpeed, float ySpeed, float zSpeed)
{
	Quaternion qRotation, qView, qNewView;

   // first create the quaternion representing the axis
   qRotation.Rotatef(AngleDir, xSpeed, ySpeed, zSpeed);

   // create view quaternion
   qView.x = xView - xPos;
   qView.y = yView - yPos;
   qView.z = zView - zPos;
   qView.w = 0;

   // calculate resulting quaternion by multiplying the rotation quaternion by the
   // view quat and finally multiply by the conjugate of the rotation quaternion
   qNewView = ((qRotation * qView) * qRotation.Conjugate());

   // update the view poing by adding the camera position and resulting quaternion
   xView = xPos + qNewView.x;
   yView = yPos + qNewView.y;
   zView = zPos + qNewView.z;
}


void Camera::RotateByMouse(int mousePosX, int mousePosY, int midX, int midY)
{
	float xDirection = 0.0f;         // Direction angle.
	float yRotation = 0.0f;          // Rotation angle.

	// If the mouseX and mouseY are at the middle of the screen then we can't rotate the view.
	if((mousePosX == midX) && (mousePosY == midY))
      return;

	// Next we get the direction of each axis.  We divide by a value to get a smaller value back.
	xDirection = (float)((midX - mousePosX)) / 20.0f;		
	yRotation = (float)((midY - mousePosY)) / 20.0f;		

	// We use curentRotX to help use keep the camera from rotating too far in either direction.
	currentRotationAngle -= yRotation;

	// Stop the camera from going to high...old value -30.0
	if(currentRotationAngle > 45.0f)
      {
		   currentRotationAngle = 45.0f;
         return;
      }

	// Stop the camera from going to low...old value -30.0
	if(currentRotationAngle < -45.0f)
		{
         currentRotationAngle = -45.0f;
         return;
      }

   // Next we get the axis which is a perpendicular vector of the view direction and up values.
   // We use the cross product of that to get the axis then we normalize it.
   float xAxis = 0, yAxis = 0, zAxis = 0;
   float xDir = 0, yDir = 0, zDir = 0;

   // Get the Direction of the view.
   xDir = xView - xPos;
   yDir = yView - yPos;
   zDir = zView - zPos;

   // Get the cross product of the direction and the up.
   xAxis = (yDir * zUp) - (zDir * yUp);
   yAxis = (zDir * xUp) - (xDir * zUp);
   zAxis = (xDir * yUp) - (yDir * xUp);

   // Normalize it.
   float len = 1 /(float)sqrt(xAxis * xAxis + yAxis * yAxis + zAxis * zAxis);
   xAxis *= len;
   yAxis *= len;
   zAxis *= len;

   // Rotate the camera.
   RotateCamera(yRotation, xAxis, yAxis, zAxis);
   RotateCamera(xDirection, 0, 1, 0);
}