// Matrix4.h: Schnittstelle fr die Klasse Matrix4
//
//////////////////////////////////////////////////////////////////////

#ifndef MATRIX4_H
#define MATRIX4_H
#include<string.h>
#include"Vector4.h"


class Matrix4x4
{
   public:
      Matrix4x4();                                // Constructor.
      Matrix4x4(const Matrix4x4& m);             // Overloaded constructors.
      Matrix4x4(float r11, float r12, float r13, float r14,
                 float r21, float r22, float r23, float r24,
                 float r31, float r32, float r33, float r34,
                 float r41, float r42, float r43, float r44);
	  Matrix4x4(const Vector4& col1, const Vector4& col2, const Vector4& col3);
	  Matrix4x4(const Vector4& col1, const Vector4& col2, const Vector4& col3, const Vector4& col4);
      ~Matrix4x4();                               // Destructor.

      void Clear();                                // Reset the matrix.
      void Zero();                                 // Reset the matrix to all 0s.

      void operator=(Matrix4x4 m);                // Overloaded = sign.
      Matrix4x4 operator-(Matrix4x4 m);          // Overloaded - sign.
      Matrix4x4 operator+(Matrix4x4 m);          // Overloaded + sign.
      Matrix4x4 operator*(Matrix4x4 m);          // Overloaded * sign.
      Matrix4x4 operator/(Matrix4x4 m);          // Overloaded / sign.

      void operator +=(Matrix4x4 m);              // Overloaded += sign.
      void operator -=(Matrix4x4 m);              // Overloaded -= sign.
      void operator *=(Matrix4x4 m);              // Overloaded *= sign.
      void operator /=(Matrix4x4 m);              // Overloaded /= sign.
      
      Matrix4x4 operator-(float f);               // Overloaded - sign.
      Matrix4x4 operator+(float f);               // Overloaded + sign.
      Matrix4x4 operator*(float f);               // Overloaded * sign.
      Matrix4x4 operator/(float f);               // Overloaded / sign.

      Vector4 operator *(Vector4 v);             // Vector4 * matrix overloaded *.

      void operator -=(float f);                   // Overloaded -= sign.
      void operator +=(float f);                   // Overloaded += sign.
      void operator *=(float f);                   // Overloaded *= sign.
      void operator /=(float f);                   // Overloaded /= sign.

      bool operator ==(Matrix4x4 v);              // Overloaded == sign.
      bool operator !=(Matrix4x4 v);              // Overloaded != sign.

      void Translate(Vector4 Translate);          // Translate a point in 3D.
      void Translate(float x, float y, float z);   // Translate a point in 3D.
      void Rotate(double angle, int x, int y,
                  int z);                          // Rotate a 3D point.
      
	  bool inverseMatrix(Matrix4x4 m);            // Inverse the current matrix data.
      bool invertMatrix(Matrix4x4 m);             // Invert a matrix.
      
	  Vector4 VectorMatrixMultiply(Vector4 v);
      Vector4 VectorMatrixMultiply3x3(Vector4 v);
      Vector4 transformPoint(Vector4 v);         // Transform a point against the matrix.

      bool CreateShadowMatrix(Vector4 planeNormal,
                              Vector4 lightPos);  // Create a shadow matrix.

      float matrix[16];                            // The 4x4 matrix in a 1D array.
};

#endif
