// Vector4.cpp: Implementierung der Klasse Vector4
//
///////////////////////////////////////////////////////////////////////

#include "Vector4.h"

#define M_PI 3.141592654


Vector4::Vector4()
{
   x = y = z = 0.0;
}


Vector4::Vector4(float X, float Y, float Z)
{
	x = X;
	y = Y;
	z = Z;
	// new
	w = 0.0f;
}


Vector4::Vector4(float X, float Y, float Z, float W)
{
	x = X;
	y = Y;
	z = Z;
	w = W;
}


Vector4::Vector4(const Vector4 &v)
{
	x = v.x;
	y = v.y;
	z = v.z;
	w = v.w;
}


void Vector4::operator =(Vector4 v)
{
	x = v.x;
	y = v.y;
	z = v.z;
	w = v.w;
}


Vector4 Vector4::operator -(Vector4 v)
{
	return Vector4(x - v.x, y - v.y, z - v.z);
}


Vector4 Vector4::operator +(Vector4 v)
{
	return Vector4(x + v.x, y + v.y, z + v.z);
}


Vector4 Vector4::operator *(Vector4 v)
{
	return Vector4(x * v.x, y * v.y, z * v.z);
}


Vector4 Vector4::operator /(Vector4 v)
{
	return Vector4(x / v.x, y / v.y, z / v.z);
}


Vector4 Vector4::operator +(float f)
{
    return Vector4(x + f, y + f, z + f);
}


Vector4 Vector4::operator -(float f)
{
    return Vector4(x - f, y - f, z - f);
}


Vector4 Vector4::operator *(float f)
{
	return Vector4(x * f, y * f, z * f);
}


Vector4 Vector4::operator /(float f)
{
	f = 1/f;
	return Vector4(x * f, y * f, z * f);
}


void Vector4::operator +=(Vector4 v)
{
	x += v.x;
	y += v.y;
	z += v.z;
}


void Vector4::operator -=(Vector4 v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
}


void Vector4::operator *=(Vector4 v)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;
}


void Vector4::operator /=(Vector4 v)
{
	x /= v.x;
	y /= v.y;
	z /= v.z;
}


void Vector4::operator +=(float f)
{
	x += f;
	y += f;
	z += f;
}


void Vector4::operator -=(float f)
{
	x -= f;
	y -= f;
	z -= f;
}


void Vector4::operator *=(float f)
{
	x *= f;
	y *= f;
	z *= f;
}


void Vector4::operator /=(float f)
{
	f = 1/f;
	x *= f;
	y *= f;
	z *= f;
}


bool Vector4::operator ==(Vector4 v)
{
	return ((x == v.x) && (y== v.y) && (z == v.z));
}


bool Vector4::operator !=(Vector4 v)
{
	return !((x == v.x) && (y== v.y) && (z == v.z));
}

void Vector4::RangeCompress(float scale)
{

    x = (x + 1.0f) * scale / 2.0f;
    y = (y + 1.0f) * scale / 2.0f;
    z = (z + 1.0f) * scale / 2.0f;
}

void Vector4::CrossProduct(Vector4 v1, Vector4 v2)
{
	x = ((v1.y * v2.z) - (v1.z * v2.y));
	y = ((v1.z * v2.x) - (v1.x * v2.z));
	z = ((v1.x * v2.y) - (v1.y * v2.x));
}


void Vector4::CrossProduct3(Vector4 v1, Vector4 v2, Vector4 v3)
{
	x = v1.y * v2.z * v3.w +
		v1.z * v2.w * v3.y +
		v1.w * v2.y * v3.z - 
		v1.y * v2.w * v3.z -
		v1.z * v2.y * v3.w -
		v1.w * v2.z * v3.y;

	y = v1.x * v2.w * v3.z +
		v1.z * v2.x * v3.w +
		v1.w * v2.z * v3.x -
		v1.x * v2.z * v3.w -
		v1.z * v2.w * v3.x -
		v1.w * v2.x * v3.z;

	z = v1.x * v2.y * v3.w +
		v1.y * v2.w * v3.x +
		v1.w * v2.x * v3.y -
		v1.x * v2.w * v3.y -
		v1.y * v2.x * v3.w -
		v1.w * v2.y * v3.x;

	w = v1.x * v2.z * v3.y +
		v1.y * v2.x * v3.z +
		v1.z * v2.y * v3.x -
		v1.x * v2.y * v3.z -
		v1.y * v2.z * v3.x -
		v1.z * v2.x * v3.y;
}


float Vector4::DotProduct3(Vector4 v1)
{
	return x * v1.x + y * v1.y + z * v1.z;
}


float Vector4::DotProduct4(Vector4 v1)
{
	return x * v1.x + y * v1.y + z * v1.z + w * v1.w;
}


float Vector4::GetLength()
{
	return (float)sqrt((x * x + y * y + z * z));
}


void Vector4::Normalize()
{
   	float lenght = GetLength();

	if(lenght == 0.0f)
		lenght = 1.0f;
	
	x = x/lenght;
	y = y/lenght;
	z = z/lenght;
	w = w/lenght;
}


void Vector4::FindTriangleNormal(Vector4 Triangle[])
{
	Vector4 v1, v2;

	v1.x = Triangle[0].x - Triangle[1].x;
	v1.y = Triangle[0].y - Triangle[1].y;
	v1.z = Triangle[0].z - Triangle[1].z;
	v1.w = Triangle[0].w - Triangle[1].w;

	v2.x = Triangle[1].x - Triangle[2].x;
	v2.y = Triangle[1].y - Triangle[2].y;
	v2.z = Triangle[1].z - Triangle[2].z;
	v2.w = Triangle[1].w - Triangle[2].w;

	CrossProduct(v1, v2);
	Normalize();
}


void Vector4::CalculateTangentVector(Vector4 Point1, Vector4 Point2, Vector4 Point3,
                                      Vector4 NormalOfA, TexCoord P1Tex, TexCoord P2Tex,
                                      TexCoord P3Tex)
{
	Vector4 VectorAB = Point2 - Point1;

   	Vector4 VectorAC = Point3 - Point1;

	// compute the components of the vectors to the vertex normal of the first point
	Vector4 ProjAB = VectorAB - (NormalOfA * NormalOfA.DotProduct4(VectorAB));
	Vector4 ProjAC = VectorAC - (NormalOfA * NormalOfA.DotProduct4(VectorAC));

	// calculate the tu difference of point 2 and 1 then of point 3 and 1
	float TexCoorUAB = P2Tex.tu - P1Tex.tu;
	float TexCoorUAC = P3Tex.tu - P1Tex.tu;

	// calculate the tv difference of point 2 and 1 then of point 3 and 1
	float TexCoorVAB = P2Tex.tv - P1Tex.tv;
	float TexCoorVAC = P3Tex.tv - P1Tex.tv;

	// switch the sign if the u of point 1 and 3 * v of 1 and 2 is greater than u of 1 and 2 *
	// v of point 1 and 3
	if((TexCoorUAC * TexCoorVAB) > (TexCoorUAB * TexCoorVAC))
    {
		TexCoorUAC = -TexCoorUAC;
        TexCoorUAB = -TexCoorUAB;
	}

	// alculate the tangent vector, normalize it, then return it (the normal values)
	Vector4 Tangent = (ProjAB * TexCoorUAC) - (ProjAC * TexCoorUAB);
	Tangent.Normalize();

	*this += Tangent;
}


void Vector4::ExtendVertexPos(Vector4 currentVertex, Vector4 lightPos, float Extend)
{
	Vector4 lightDir;  // direction of the light to the vertex position
    Vector4 newPos;    // new extended vertex position to make up the shadow volume

    // get the light direction from the vertex position and the light position
    lightDir = currentVertex - lightPos;

    // now that we know where its going we add it to the position of the light so
    // we get the correct, new position; we multiply it by a passed it value to
    // give the volume some distance or things won't come out quite as we want
    newPos = lightPos + lightDir * Extend;

    x = newPos.x;
    y = newPos.y;
    z = newPos.z;
    w = newPos.w;
}


void Vector4::ExtendVertexPos(Vector4 lightPos, float Extend)
{
	Vector4 lightDir;  // direction of the light to the vertex position
    Vector4 newPos;    // hew extended vertex position to make up the shadow volume

    // get the light direction from the vertex position and the light position
    lightDir = Vector4(x, y, z, w) - lightPos;

    // now that we know where its going we add it to the position of the light so
    // we get the correct, new position; we multiply it by a passed it value to
    // give the volume some distance or things won't come out quite as we want
    newPos = lightPos + lightDir * Extend;

    x = newPos.x;
    y = newPos.y;
    z = newPos.z;
    w = newPos.w;
}


Vector4 Vector4::GetRotatedX(double angle)
{
	float sinAngle = (float)sin(M_PI * angle / 180);
	float cosAngle = (float)cos(M_PI * angle / 180);

	return Vector4(x, y * cosAngle - z * sinAngle, y * sinAngle + z * cosAngle, w);
}


Vector4 Vector4::GetRotatedY(double angle)
{
	float sinAngle = (float)sin(M_PI * angle / 180);
	float cosAngle = (float)cos(M_PI * angle / 180);

	return Vector4(x * cosAngle + z * sinAngle, y, -x * sinAngle + z * cosAngle, w);
}


Vector4 Vector4::GetRotatedZ(double angle)
{
	float sinAngle = (float)sin(M_PI * angle / 180);
	float cosAngle = (float)cos(M_PI * angle / 180);

	return Vector4(x * cosAngle - y * sinAngle, x * sinAngle + y * cosAngle, z, w);
}


Vector4 Vector4::GetRotatedAxis(double angle, Vector4 axis)
{
	if(angle == 0.0) return(*this);

	axis.Normalize();

	Vector4 RotationRow1, RotationRow2, RotationRow3;

	double newAngle = M_PI * angle / 180;
	float sinAngle = (float)sin(newAngle);
	float cosAngle = (float)cos(newAngle);
	float oneSubCos = 1.0f - cosAngle;

	RotationRow1.x = (axis.x) * (axis.x) + cosAngle * (1 - (axis.x) * (axis.x));
	RotationRow1.y = (axis.x) * (axis.y) * (oneSubCos) - sinAngle * axis.z;
	RotationRow1.z = (axis.x) * (axis.z) * (oneSubCos) + sinAngle * axis.y;

	RotationRow2.x = (axis.x) * (axis.y) * (oneSubCos) + sinAngle * axis.z;
	RotationRow2.y = (axis.y) * (axis.y) + cosAngle * (1 - (axis.y) * (axis.y));
	RotationRow2.z = (axis.y) * (axis.z) * (oneSubCos) - sinAngle * axis.x;
	
	RotationRow3.x = (axis.x) * (axis.z) * (oneSubCos) - sinAngle * axis.y;
	RotationRow3.y = (axis.y) * (axis.z) * (oneSubCos) + sinAngle * axis.x;
	RotationRow3.z = (axis.z) * (axis.z) + cosAngle * (1 - (axis.z) * (axis.z));

	return Vector4(this->DotProduct3(RotationRow1),
                   this->DotProduct3(RotationRow2),
                   this->DotProduct3(RotationRow3));
}


void Vector4::CalculateBinormalVector(Vector4 tangent, Vector4 normal)
{
   this->CrossProduct(tangent, normal);
}


void Vector4::ClampTo01()
{
   Vector4 temp(x, y, z, w);
   temp.Normalize();
   temp = temp * 0.5f + Vector4(0.5f, 0.5f, 0.5f);

   x = temp.x;
   y = temp.y;
   z = temp.z;
}