// Vector4.h: Schnittstelle fr die Klasse Vector4
//
//////////////////////////////////////////////////////////////////////

#ifndef VECTOR4_H
#define VECTOR4_H

#include <math.h>


class TexCoord
{
   public:
      TexCoord()
         {
            tu = 0.0f; tv = 0.0f;
         }

      TexCoord(float u, float v)
         {
            tu = u; tv = v;
         }

      float tu, tv;
};


class Vector4
{
	public:
		// constructors/destructors
		Vector4();
		Vector4(float X, float Y, float Z);
		Vector4(float X, float Y, float Z, float W);
		Vector4(const Vector4 &v);

		// operators
		void operator=(Vector4 v);
		Vector4 operator-(Vector4 v);
		Vector4 operator+(Vector4 v);
		Vector4 operator*(Vector4 v);
		Vector4 operator/(Vector4 v);
		Vector4 operator+(float f);
		Vector4 operator-(float f);
		Vector4 operator*(float f);
		Vector4 operator/(float f);

		// signs
	    void operator +=(Vector4 v);
		void operator -=(Vector4 v);
		void operator *=(Vector4 v);
		void operator /=(Vector4 v); 
		void operator +=(float f);
		void operator -=(float f);
		void operator *=(float f);
		void operator /=(float f);

		bool operator ==(Vector4 v);
		bool operator !=(Vector4 v);

		// methods
		void RangeCompress(float scale);
		void CrossProduct(Vector4 v1, Vector4 v2); // stores cross product of v1 and v2
		void CrossProduct3(Vector4 v1, Vector4 v2, Vector4 v3); // stores cross product of v1, v2, v3
		float DotProduct3(Vector4 v1); // dot product 3 on v1 and this
		float DotProduct4(Vector4 v1); // returns dot product 4 of v1 and this
		float GetLength();
		void Normalize(); // normalize this vector
		void FindTriangleNormal(Vector4 Triangle[]); // find the normal of a triangle.
		void CalculateTangentVector(Vector4 Point1,
									Vector4 Point2,
									Vector4 Point3,
									Vector4 NormalOfA,
									TexCoord P1Tex,
									TexCoord P2Tex,
									TexCoord P3Tex);      // cacluate tagent vector & store here
		void ExtendVertexPos(Vector4 lightPos, float Extend); // extend point based on light position and currend point & store here
		void ExtendVertexPos(Vector4 currentVertex, Vector4 lightPos, float Extend); // takes current point and source and extend & store here
		Vector4 GetRotatedX(double angle); // rotate along x
		Vector4 GetRotatedY(double angle); // rotate alone y
		Vector4 GetRotatedZ(double angle); // rotate along z
		Vector4 GetRotatedAxis(double angle, Vector4 axis); // rotate along an arbitrary axis
		void CalculateBinormalVector(Vector4 tangent, Vector4 normal); // get binormal
		void ClampTo01();	// clamp this vector between 0 and 1

		// fields
		float x, y, z, w;
};

#endif